export class AudioPromiseInner {
  async play(src, startTime = 0, endTime = undefined, timeUpdate = f => f, updateStatus = f => f) {
    this.timeUpdateCB = timeUpdate
    if (!src) throw new Error('bad audio src: ', src)
    let r = await window.weixinReadyPromise
    if (!this.audio) {
      this.audio = new Audio()
      this.audio.addEventListener("canplay", () => this.onCanPlay())
      this.audio.addEventListener("ended", () => this.onEnded())
      this.audio.addEventListener("timeupdate", e => this.onTimeupdate(e))
      for (let e of "abort error stalled suspend waiting".split(' ')) {
        console.log('adding listener: ', e)
        this.audio.addEventListener(e, console.log)
      }
    }
    if (this.src !== src) {
      if (this.src) {
        this.pause()
      }
      this.updateStatus = updateStatus
      this.src = src
      this.audio.src = src
      this.audio.load()
      this.canPlayPromise = new Promise((resolve, reject) => {
        this.canPlayDefer = {resolve, reject}
      })
    }
    console.log(`playing ${this.src} from ${startTime} -> ${endTime || ''}`)
    // alert('before canplay')
    await this.canPlayPromise
    // alert('after canplay')
    return new Promise((resolve, reject) => {
      this.defer = {resolve, reject}
      this.endTime = endTime
      this.audio.currentTime = startTime
      this.audio.play().catch(() => { // 错误的情况也要让播放返回
        resolve(false)
      })
    })
  }
  pause(src) {
    if (src && src !== this.src) {
      return
    }
    this.audio && this.audio.pause()
    this.updateStatus && this.updateStatus('paused')
    if (this.defer) {
      this.defer.resolve(true)
    }
  }
  onEnded() {
    this.pause()
  }
  onTimeupdate(e) {
    if (this.endTime && +e.target.currentTime > this.endTime) {
      this.pause()
    }
    this.timeUpdateCB(e)
  }
  onCanPlay() {
    this.canPlayDefer.resolve(true)
  }
}

let singleAudio = new AudioPromiseInner('')

export class AudioPromise {
  constructor(src, timeUpdate = f => f, updateStatus = f => f) {
    this.src = src
    this.timeUpdate = timeUpdate
    this.updateStatus = updateStatus
  }
  async play(startTime = 0, endTime = undefined) {
    return singleAudio.play(this.src, startTime, endTime, this.timeUpdate, this.updateStatus)
  }
  async pause() {
    return singleAudio.pause(this.src)
  }
}
